# -*- coding: utf-8 -*-

__author__ = 'willmcginnis'
__all__ = ['SparkConf']


class SparkConf(object):

    DEBUG_STRING = 'no string for dummy version'

    def __init__(self, loadDefaults=True, _jvm=None, _jconf=None):
        """

        :param loadDefaults:
        :param _jvm:
        :param _jconf:
        :return:
        """
        self.conf = {}

    def set(self, key, value):
        """

        :param key:
        :param value:
        :return:
        """
        self.conf.update({key: value})
        return self

    def setIfMissing(self, key, value):
        """

        :param key:
        :param value:
        :return:
        """
        v = self.conf.get(key, None)
        if v is None:
            self.set(key, value)

        return self

    def setMaster(self, value):
        """

        :param value:
        :return:
        """
        self.set('master', value)
        return self

    def setAppName(self, value):
        """

        :param value:
        :return:
        """
        self.set('appName', value)
        return self

    def setSparkHome(self, value):
        """

        :param value:
        :return:
        """
        self.set('sparkHome', value)
        return self

    def setExecutorEnv(self, key=None, value=None, pairs=None):
        """

        :param key:
        :param value:
        :param pairs:
        :return:
        """
        if key is not None and value is not None:
            self.set(key, value)

        if pairs is not None:
            for k, v in pairs:
                self.set(k, v)

        return self

    def setAll(self, pairs):
        """

        :param pairs:
        :return:
        """
        if pairs is not None:
            for k, v in pairs:
                self.set(k, v)

        return self

    def get(self, key, defaultValue=None):
        """

        :param key:
        :param defaultValue:
        :return:
        """
        return self.conf.get(key, defaultValue)

    def getAll(self):
        """

        :return:
        """
        return [(k, v) for k, v in self.conf.items()]

    def contains(self, key):
        """

        :param key:
        :return:
        """
        return key in self.conf.keys()

    def toDebugString(self):
        """

        :return:
        """
        return self.DEBUG_STRING



# -*- coding: utf-8 -*-

import os
import random

from dummy_spark import SparkContext, SparkConf
from dummy_spark.sql import SQLContext

__author__ = 'willmcginnis'

# make a spark conf
sconf = SparkConf()

# set some property (won't do anything)
sconf.set('spark.executor.extraClassPath', 'foo')

# use the spark conf to make a spark context
sc = SparkContext(master='', conf=sconf)

# set the log level (also doesn't do anything)
sc.setLogLevel('INFO')

# maybe make a useless sqlcontext (nothing implimented here yet)
sqlctx = SQLContext(sc)

# add pyfile just appends to the sys path
sc.addPyFile(os.path.dirname(__file__))

# do some hadoop configuration into the ether
sc._jsc.hadoopConfiguration().set('foo', 'bar')

# maybe make some data
rdd = sc.parallelize([1, 2, 3, 4, 5])

# map and collect
print('\nmap()')
rdd = rdd.map(lambda x: x ** 2)
print(rdd.collect())

# add some more in there
print('\nunion()')
rdd2 = sc.parallelize([2, 4, 10])
rdd = rdd.union(rdd2)
print(rdd.collect())

# filter and take
print('\nfilter()')
rdd = rdd.filter(lambda x: x > 4)
print(rdd.take(10))

# flatmap
print('\nflatMap()')
rdd = rdd.flatMap(lambda x: [x, x, x])
print(rdd.collect())

# group by key
print('\ngroupByKey()')
rdd = rdd.map(lambda x: (x, random.random()))
rdd = rdd.groupByKey()
print(rdd.collect())
rdd = rdd.mapValues(list)
print(rdd.collect())

# forEachPartition
print('\nforEachPartition()')
rdd.foreachPartition(lambda x: print('partition: ' + str(x)))

# reducebykey
print('\nReduceByKey')
rdd = sc.parallelize([(0, 'A'), (0, 'A'), (1, 'C')])
rdd = rdd.reduceByKey(lambda a, b: a)
print(rdd)


# -*- coding: utf-8 -*-

from __future__ import print_function

import gzip
import time
import sys

if sys.version_info.major == 3:
    from io import StringIO
elif sys.version_info.major == 2:
    try:
        import cStringIO as StringIO
    except ImportError:
        import StringIO
else:
    raise ValueError('Python %s not supported' % (sys.version_info.major, ))

from threading import Lock
from dummy_spark.rdd import RDD

try:
    import tinys3
    has_tinys3 = True
except ImportError as e:
    has_tinys3 = False


__all__ = ['SparkContext']
__author__ = 'willmcginnis'


class hadoopConfiguration(object):
    def __init__(self):
        pass

    def set(self, a, b):
        setattr(self, a, b)
        return True

    def get(self, a):
        return getattr(self, a, None)


class jvm(object):
    def __init__(self):
        self.hc = hadoopConfiguration()

    def hadoopConfiguration(self):
        return self.hc

    def textFile(self, file_name):
        if file_name.startswith('s3'):
            if has_tinys3:
                file_name = file_name.split('://')[1]
                bucket_name = file_name.split('/')[0]
                key_name = file_name.replace(bucket_name, '')[1:]
                access_key = self.hc.get('fs.s3n.awsAccessKeyId')
                secret_key = self.hc.get('fs.s3n.awsSecretAccessKey')
                conn = tinys3.Connection(access_key, secret_key)
                file = conn.get(key_name, bucket_name)
                if file_name.endswith('.gz'):
                    compressed = StringIO.StringIO(file.content)
                    gzipper = gzip.GzipFile(fileobj=compressed) 
                    return gzipper.readlines()
                return file.content.decode('utf-8').split('\n')
            else:
                raise Exception('Need TinyS3 to use s3 files')
        else:
            if file_name.endswith('.gz'):
                opener = gzip.open
            else:
                opener = open
            with opener(file_name, 'r') as f:
                return f.readlines()


class SparkContext(object):
    """
    Main entry point for Spark functionality. A SparkContext represents the
    connection to a Spark cluster, and can be used to create L{RDD} and
    broadcast variables on that cluster.
    """

    _gateway = None
    _jvm = None
    _next_accum_id = 0
    _active_spark_context = None
    _lock = Lock()
    _jsc = jvm()
    _python_includes = None  # zip and egg files that need to be added to PYTHONPATH

    PACKAGE_EXTENSIONS = ('.zip', '.egg', '.jar')

    DUMMY_VERSION = 'dummy version'

    def __init__(self, master=None, appName=None, sparkHome=None, pyFiles=None, environment=None, batchSize=0, serializer=None, conf=None, gateway=None, jsc=None, profiler_cls=None):
        """

        :param master:
        :param appName:
        :param sparkHome:
        :param pyFiles:
        :param environment:
        :param batchSize:
        :param serializer:
        :param conf:
        :param gateway:
        :param jsc:
        :param profiler_cls:
        :return:
        """
        self._callsite = None
        SparkContext._ensure_initialized(self, gateway=gateway)
        self.start_time = time.time()
        try:
            self._do_init(master, appName, sparkHome, pyFiles, environment, batchSize, serializer, conf, jsc, profiler_cls)
        except:
            # If an error occurs, clean up in order to allow future SparkContext creation:
            self.stop()
            raise

    def _do_init(self, master, appName, sparkHome, pyFiles, environment, batchSize, serializer, conf, jsc, profiler_cls):
        """

        :param master:
        :param appName:
        :param sparkHome:
        :param pyFiles:
        :param environment:
        :param batchSize:
        :param serializer:
        :param conf:
        :param jsc:
        :param profiler_cls:
        :return:
        """
        # TODO: add included files to python path
        return

    def _initialize_context(self, jconf):
        """

        :param jconf:
        :return:
        """
        return None

    @classmethod
    def _ensure_initialized(cls, instance=None, gateway=None):
        """

        :param instance:
        :param gateway:
        :return:
        """
        return True

    def __enter__(self):
        """

        :return:
        """
        return self

    def __exit__(self, type, value, trace):
        """

        :param type:
        :param value:
        :param trace:
        :return:
        """
        self.stop()

    def setLogLevel(self, logLevel):
        """

        :param logLevel:
        :return:
        """
        pass

    @classmethod
    def setSystemProperty(cls, key, value):
        """

        :param key:
        :param value:
        :return:
        """
        pass

    @property
    def version(self):
        """

        :return:
        """
        return self.DUMMY_VERSION

    @property
    def startTime(self):
        """

        :return:
        """
        return self.start_time

    @property
    def defaultParallelism(self):
        """

        :return:
        """
        return 1

    @property
    def defaultMinPartitions(self):
        """

        :return:
        """
        return 1

    def stop(self):
        """

        :return:
        """
        pass

    def emptyRDD(self):
        """

        :return:
        """
        return RDD([], self, None)

    def range(self, start, end=None, step=1, numSlices=None):
        """

        :param start:
        :param end:
        :param step:
        :param numSlices:
        :return:
        """
        return RDD(list(range(start, end, step)), self, None)

    def parallelize(self, c, numSlices=None):
        """

        :param c:
        :param numSlices:
        :return:
        """
        return RDD(c, self, None)

    def textFile(self, name, minPartitions=None, use_unicode=True):
        """

        :param name:
        :param minPartitions:
        :param use_unicode:
        :return:
        """
        data = self._jsc.textFile(name)
        rdd = RDD(data, self, None)
        return rdd

    def addPyFile(self, path):
        """

        :param path:
        :return:
        """
        sys.path.append(path)

    def pickleFile(self, name, minPartitions=None):
        """
        NotImplemented

        :param name:
        :param minPartitions:
        :return:
        """

        raise NotImplementedError

    def wholeTextFiles(self, path, minPartitions=None, use_unicode=True):
        """
        NotImplemented

        :param path:
        :param minPartitions:
        :param use_unicode:
        :return:
        """
        raise NotImplementedError

    def binaryFiles(self, path, minPartitions=None):
        """
        NotImplemented

        :param path:
        :param minPartitions:
        :return:
        """
        raise NotImplementedError

    def binaryRecords(self, path, recordLength):
        """
        NotImplemented

        :param path:
        :param recordLength:
        :return:
        """
        raise NotImplementedError

    def _dictToJavaMap(self, d):
        """
        NotImplemented

        :param d:
        :return:
        """
        raise NotImplementedError

    def sequenceFile(self, path, keyClass=None, valueClass=None, keyConverter=None, valueConverter=None, minSplits=None, batchSize=0):
        """
        NotImplemented

        :param path:
        :param keyClass:
        :param valueClass:
        :param keyConverter:
        :param valueConverter:
        :param minSplits:
        :param batchSize:
        :return:
        """
        raise NotImplementedError

    def newAPIHadoopFile(self, path, inputFormatClass, keyClass, valueClass, keyConverter=None, valueConverter=None, conf=None, batchSize=0):
        """
        NotImplemented

        :param path:
        :param inputFormatClass:
        :param keyClass:
        :param valueClass:
        :param keyConverter:
        :param valueConverter:
        :param conf:
        :param batchSize:
        :return:
        """
        raise NotImplementedError

    def newAPIHadoopRDD(self, inputFormatClass, keyClass, valueClass, keyConverter=None, valueConverter=None, conf=None, batchSize=0):
        """
        Read a 'new API' Hadoop InputFormat with arbitrary key and value class, from an arbitrary Hadoop configuration,
        which is passed in as a Python dict. This will be converted into a Configuration in Java. The mechanism is the
        same as for sc.sequenceFile.

        :param inputFormatClass: fully qualified classname of Hadoop InputFormat (e.g. "org.apache.hadoop.mapreduce.lib.input.TextInputFormat")
        :param keyClass: fully qualified classname of key Writable class (e.g. "org.apache.hadoop.io.Text")
        :param valueClass: fully qualified classname of value Writable class (e.g. "org.apache.hadoop.io.LongWritable")
        :param keyConverter: (None by default)
        :param valueConverter: (None by default)
        :param conf: Hadoop configuration, passed in as a dict (None by default)
        :param batchSize: The number of Python objects represented as a single Java object. (default 0, choose batchSize automatically)
        """

        if 'elasticsearch' in inputFormatClass and 'elasticsearch' in valueClass:
            try:
                from elasticsearch import Elasticsearch
            except ImportError:
                raise ImportError('Must have elasticsearch-py installed to use NewAPIHadoopRDD with the elasticsearch driver')

            host_name = conf.get('es.nodes')
            host_port = conf.get('es.port')
            index, mapping = conf.get('es.resource', '/').split('/')
            query = conf.get('es.query')

            client = Elasticsearch(hosts=['http://%s:%s' % (host_name, host_port, )])

            data = client.search(index=index, doc_type=mapping, body=query)
            data = data.get('hits', {}).get('hits', [])

            cleaned_data = []
            for dat in data:
                if '_source' in dat.keys():
                    cleaned_data.append((dat.get('_id'), dat.get('_source', {})))
                elif 'fields' in dat.keys():
                    cleaned_data.append((dat.get('_id'), dat.get('fields')))

            rdd = RDD(cleaned_data, self, None)

            return rdd
        else:
            raise NotImplementedError('Have not implimented %s for NewAPIHadoopRDD' % (inputFormatClass, ))

    def hadoopFile(self, path, inputFormatClass, keyClass, valueClass, keyConverter=None, valueConverter=None, conf=None, batchSize=0):
        """
        NotImplemented

        :param path:
        :param inputFormatClass:
        :param keyClass:
        :param valueClass:
        :param keyConverter:
        :param valueConverter:
        :param conf:
        :param batchSize:
        :return:
        """
        raise NotImplementedError

    def hadoopRDD(self, inputFormatClass, keyClass, valueClass, keyConverter=None, valueConverter=None, conf=None, batchSize=0):
        """
        NotImplemented

        :param inputFormatClass:
        :param keyClass:
        :param valueClass:
        :param keyConverter:
        :param valueConverter:
        :param conf:
        :param batchSize:
        :return:
        """
        raise NotImplementedError

    def _checkpointFile(self, name, input_deserializer):
        """
        NotImplemented

        :param name:
        :param input_deserializer:
        :return:
        """
        raise NotImplementedError

    def union(self, rdds):
        """
        NotImplemented

        :param rdds:
        :return:
        """
        raise NotImplementedError

    def broadcast(self, value):
        """
        NotImplemented

        :param value:
        :return:
        """
        raise NotImplementedError

    def accumulator(self, value, accum_param=None):
        """
        NotImplemented

        :param value:
        :param accum_param:
        :return:
        """
        raise NotImplementedError

    def addFile(self, path):
        """
        NotImplemented

        :param path:
        :return:
        """
        raise NotImplementedError

    def clearFiles(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def setCheckpointDir(self, dirName):
        """
        NotImplemented

        :param dirName:
        :return:
        """
        raise NotImplementedError

    def _getJavaStorageLevel(self, storageLevel):
        """
        NotImplemented

        :param storageLevel:
        :return:
        """
        raise NotImplementedError

    def setJobGroup(self, groupId, description, interruptOnCancel=False):
        """
        NotImplemented

        :param groupId:
        :param description:
        :param interruptOnCancel:
        :return:
        """
        raise NotImplementedError

    def setLocalProperty(self, key, value):
        """
        NotImplemented

        :param key:
        :param value:
        :return:
        """
        raise NotImplementedError

    def getLocalProperty(self, key):
        """
        NotImplemented

        :param key:
        :return:
        """
        raise NotImplementedError

    def sparkUser(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def cancelJobGroup(self, groupId):
        """
        NotImplemented

        :param groupId:
        :return:
        """
        raise NotImplementedError

    def cancelAllJobs(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def statusTracker(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def runJob(self, rdd, partitionFunc, partitions=None, allowLocal=False):
        """
        NotImplemented

        :param rdd:
        :param partitionFunc:
        :param partitions:
        :param allowLocal:
        :return:
        """
        raise NotImplementedError

    def show_profiles(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def dump_profiles(self, path):
        """
        NotImplemented

        :param path:
        :return:
        """
        raise NotImplementedError



# -*- coding: utf-8 -*-

from dummy_spark.conf import SparkConf
from dummy_spark.context import SparkContext
from dummy_spark.rdd import RDD

__author__ = 'willmcginnis'

__all__ = [
    'RDD',
    'SparkConf',
    'SparkContext',
]



# -*- coding: utf-8 -*-

import random
import uuid
from functools import reduce
from dummy_spark.resultsiterable import ResultIterable

__author__ = 'willmcginnis'


class RDD(object):
    """
    A Resilient Distributed Dataset (RDD) is the basic abstraction in Spark. It represents an immutable, partitioned
    collection of elements that can be operated on in parallel. This is a dummy version of that, that is just a list
    under the hood.  To be used for testing, and maybe development if you play fast and loose.

    Important note: the dummy RDD is NOT lazily loaded.

    """

    def __init__(self, jrdd, ctx, jrdd_deserializer=None):
        """

        :param jrdd:
        :param ctx:
        :param jrdd_deserializer:
        :return:
        """
        # ported
        self._id = str(uuid.uuid4())

        if jrdd is None:
            self._jrdd = []
        else:
            if isinstance(jrdd, list):
                self._jrdd = jrdd
            elif isinstance(jrdd, set):
                self._jrdd = list(jrdd)
            else:
                raise AttributeError('Type %s for jrdd not supported' % (type(jrdd), ))

        self.ctx = ctx
        self.is_cached = True
        self._name = 'dummpy-rdd'

        # not ported

        self.is_checkpointed = False

        self._jrdd_deserializer = jrdd_deserializer
        self.partitioner = None

    def id(self):
        """

        :return:
        """
        return self._id

    @property
    def context(self):
        """

        :return:
        """
        return self.ctx

    def name(self):
        """

        :return:
        """
        return self._name

    def setName(self, name):
        """

        :param name:
        :return:
        """
        self._name = name
        return self

    def __repr__(self):
        """

        :return:
        """
        return str(self._jrdd)

    def cache(self):
        """

        :return:
        """
        return self

    def persist(self, storageLevel=None):
        """

        :param storageLevel:
        :return:
        """
        return self

    def unpersist(self):
        """

        :return:
        """
        return self

    def _reserialize(self, serializer=None):
        """

        :param serializer:
        :return:
        """
        return self

    def checkpoint(self):
        """

        :return:
        """
        pass

    def isCheckpointed(self):
        """

        :return:
        """
        return True

    def getCheckpointFile(self):
        """

        :return:
        """
        return None

    def map(self, f, preservesPartitioning=False):
        """

        :param f:
        :param preservesPartitioning:
        :return:
        """
        data = list(map(f, self._jrdd))
        return RDD(data, self.ctx)

    def flatMap(self, f, preservesPartitioning=False):
        """

        :param f:
        :param preservesPartitioning:
        :return:
        """
        data = [item for sl in map(f, self._jrdd) for item in sl]
        return RDD(data, self.ctx)

    def mapPartitions(self, f, preservesPartitioning=False):
        """

        :param f:
        :param preservesPartitioning:
        :return:
        """
        return self.map(f, preservesPartitioning=preservesPartitioning)

    def getNumPartitions(self):
        """

        :return:
        """
        return 1

    def filter(self, f):
        """

        :param f:
        :return:
        """
        data = list(filter(f, self._jrdd))
        return RDD(data, self.ctx)

    def distinct(self, numPartitions=None):
        """

        :param numPartitions:
        :return:
        """
        data = set(self._jrdd)
        return RDD(data, self.ctx)

    def sample(self, withReplacement, fraction, seed=None):
        """

        :param withReplacement:
        :param fraction:
        :param seed:
        :return:
        """
        assert fraction >= 0.0, "Negative fraction value: %s" % fraction

        if seed is not None:
            random.seed(seed)

        idx_list = list(range(len(self._jrdd)))
        if withReplacement:
            data = [self._jrdd[random.choice(idx_list)] for _ in list(range(int(fraction * len(self._jrdd))))]
        else:
            random.shuffle(idx_list)
            data = [self._jrdd[idx] for idx in idx_list[:int(fraction * len(self._jrdd))]]
        return RDD(data, self.ctx)

    def randomSplit(self, weights, seed=None):
        """

        :param weights:
        :param seed:
        :return:
        """
        pass

    def takeSample(self, withReplacement, num, seed=None):
        """

        :param withReplacement:
        :param num:
        :param seed:
        :return:
        """
        assert num >= 0.0, "Negative sample num: %s" % num

        if seed is not None:
            random.seed(seed)

        if withReplacement:
            out = [self._jrdd[random.choice(list(range(len(self._jrdd))))] for _ in num]
        else:
            idx_list = list(range(len(self._jrdd)))
            random.shuffle(idx_list)
            out = [self._jrdd[idx] for idx in idx_list[:num]]
        return out

    def union(self, other):
        """

        :param other:
        :return:
        """
        return RDD(self._jrdd + other._jrdd, self.ctx)

    def intersection(self, other):
        """

        :param other:
        :return:
        """
        data = [item for item in self._jrdd if item in other._jrdd]
        return RDD(data, self.ctx)

    def __add__(self, other):
        """

        :param other:
        :return:
        """
        if not isinstance(other, RDD):
            raise TypeError
        return self.union(other)

    def repartitionAndSortWithinPartitions(self, numPartitions=None, partitionFunc=None, ascending=True, keyfunc=lambda x: x):
        """

        :param numPartitions:
        :param partitionFunc:
        :param ascending:
        :param keyfunc:
        :return:
        """
        data = sorted(self._jrdd, key=keyfunc, reverse=ascending)
        return RDD(data, self.ctx)

    def sortByKey(self, ascending=True, numPartitions=None, keyfunc=lambda x: x):
        """

        :param ascending:
        :param numPartitions:
        :param keyfunc:
        :return:
        """
        data = sorted(self._jrdd, key=keyfunc, reverse=ascending)
        return RDD(data, self.ctx)

    def sortBy(self, keyfunc, ascending=True, numPartitions=None):
        """

        :param keyfunc:
        :param ascending:
        :param numPartitions:
        :return:
        """
        data = sorted(self._jrdd, key=keyfunc, reverse=ascending)
        return RDD(data, self.ctx)

    def glom(self):
        """

        :return:
        """
        return self._jrdd

    def cartesian(self, other):
        """

        :param other:
        :return:
        """
        data = [(t, u) for t in self._jrdd for u in other._jrdd]
        return RDD(data, self.ctx)

    def groupBy(self, f, numPartitions=None):
        """

        :param f:
        :param numPartitions:
        :return:
        """
        return self.map(lambda x: (f(x), x)).groupByKey(numPartitions)

    def foreach(self, f):
        """

        :param f:
        :return:
        """
        return self.map(f)

    def foreachPartition(self, f):
        """

        :param f:
        :return:
        """
        return f(self._jrdd)

    def collect(self):
        """

        :return:
        """
        return self._jrdd

    def sum(self):
        """

        :return:
        """
        return sum(self._jrdd)

    def count(self):
        """

        :return:
        """
        return len(self._jrdd)

    def mean(self):
        """

        :return:
        """
        return float(sum(self._jrdd)) / len(self._jrdd)

    def take(self, num):
        """

        :param num:
        :return:
        """
        return self._jrdd[:num]

    def first(self):
        """

        :return:
        """
        return self._jrdd[0]

    def isEmpty(self):
        """

        :return:
        """
        return len(self._jrdd) == 0

    def reduceByKey(self, func, numPartitions=None):
        """

        :param func:
        :param numPartitions:
        :return:
        """
        keys = {kv[0] for kv in self._jrdd}
        data = [(key, reduce(func, [kv[1] for kv in self._jrdd if kv[0] == key])) for key in keys]
        return RDD(data, self.ctx)

    # TODO: support variant with custom partitioner
    def groupByKey(self, numPartitions=None):
        """

        :param numPartitions:
        :return:
        """
        keys = {x[0] for x in self._jrdd}
        out = {k: ResultIterable([x[1] for x in self._jrdd if x[0] == k]) for k in keys}
        data = list(out.items())
        return RDD(data, self.ctx)

    def flatMapValues(self, f):
        """

        :param f:
        :return:
        """
        flat_map_fn = lambda kv: ((kv[0], x) for x in f(kv[1]))
        return self.flatMap(flat_map_fn, preservesPartitioning=True)

    def mapValues(self, f):
        """

        :param f:
        :return:
        """
        map_values_fn = lambda kv: (kv[0], f(kv[1]))
        return self.map(map_values_fn, preservesPartitioning=True)

    def cogroup(self, other, numPartitions=None):
        """

        :param other:
        :param numPartitions:
        :return:
        """
        vs = {x[0] for x in self._jrdd}
        us = {x[0] for x in other._jrdd}
        keys = vs.union(us)
        data = [
            (
                k,
                ([v[1] for v in self._jrdd if v[0] == k]),
                ([u[1] for u in other._jrdd if u[0] == k])
            )
            for k in keys
        ]
        return RDD(data, self.ctx)

    def zip(self, other):
        """

        :param other:
        :return:
        """
        data = list(zip(other, self._jrdd))
        return RDD(data, self.ctx)

    def zipWithIndex(self):
        """

        :return:
        """
        data = [(b, a) for a, b in list(enumerate(self._jrdd))]
        return RDD(data, self.ctx)

    def _defaultReducePartitions(self):
        """

        :return:
        """
        return 1

    def lookup(self, key):
        """

        :param key:
        :return:
        """
        return [x for x in self._jrdd if x[0] == key]

    def countApprox(self, timeout, confidence=0.95):
        """

        :param timeout:
        :param confidence:
        :return:
        """
        return len(self._jrdd)

    def sumApprox(self, timeout, confidence=0.95):
        """

        :param timeout:
        :param confidence:
        :return:
        """
        return sum(self._jrdd)

    def meanApprox(self, timeout, confidence=0.95):
        """

        :param timeout:
        :param confidence:
        :return:
        """
        return float(sum(self._jrdd)) / len(self._jrdd)

    def countApproxDistinct(self, relativeSD=0.05):
        """

        :param relativeSD:
        :return:
        """
        return len(set(self._jrdd))

    def toLocalIterator(self):
        """

        :return:
        """
        for row in self._jrdd:
            yield row

    def max(self, key=None):
        """

        :param key:
        :return:
        """
        if key is None:
            return max(self._jrdd)
        else:
            raise NotImplementedError

    def min(self, key=None):
        """

        :param key:
        :return:
        """
        if key is None:
            return min(self._jrdd)
        else:
            raise NotImplementedError

    def _pickled(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def mapPartitionsWithIndex(self, f, preservesPartitioning=False):
        """
        NotImplemented

        :param f:
        :param preservesPartitioning:
        :return:
        """
        raise NotImplementedError

    @staticmethod
    def _computeFractionForSampleSize(sampleSizeLowerBound, total, withReplacement):
        """
        NotImplemented

        :param sampleSizeLowerBound:
        :param total:
        :param withReplacement:
        :return:
        """
        raise NotImplementedError

    def pipe(self, command, env=None):
        """
        NotImplemented

        :param command:
        :param env:
        :return:
        """
        raise NotImplementedError

    def reduce(self, f):
        """
        NotImplemented

        :param f:
        :return:
        """
        raise NotImplementedError

    def treeReduce(self, f, depth=2):
        """
        NotImplemented

        :param f:
        :param depth:
        :return:
        """
        raise NotImplementedError

    def fold(self, zeroValue, op):
        """
        NotImplemented

        :param zeroValue:
        :param op:
        :return:
        """
        raise NotImplementedError

    def aggregate(self, zeroValue, seqOp, combOp):
        """
        NotImplemented

        :param zeroValue:
        :param seqOp:
        :param combOp:
        :return:
        """
        raise NotImplementedError

    def treeAggregate(self, zeroValue, seqOp, combOp, depth=2):
        """
        NotImplemented

        :param zeroValue:
        :param seqOp:
        :param combOp:
        :param depth:
        :return:
        """
        raise NotImplementedError

    def stats(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def histogram(self, buckets):
        """
        NotImplemented

        :param buckets:
        :return:
        """
        raise NotImplementedError

    def variance(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def stdev(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def sampleStdev(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def sampleVariance(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def countByValue(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def top(self, num, key=None):
        """
        NotImplemented

        :param num:
        :param key:
        :return:
        """
        raise NotImplementedError

    def takeOrdered(self, num, key=None):
        """
        NotImplemented

        :param num:
        :param key:
        :return:
        """
        raise NotImplementedError

    def saveAsNewAPIHadoopDataset(self, conf, keyConverter=None, valueConverter=None):
        """
        NotImplemented

        :param conf:
        :param keyConverter:
        :param valueConverter:
        :return:
        """
        raise NotImplementedError

    def saveAsNewAPIHadoopFile(self, path, outputFormatClass, keyClass=None, valueClass=None, keyConverter=None, valueConverter=None, conf=None):
        """
        NotImplemented

        :param path:
        :param outputFormatClass:
        :param keyClass:
        :param valueClass:
        :param keyConverter:
        :param valueConverter:
        :param conf:
        :return:
        """
        raise NotImplementedError

    def saveAsHadoopDataset(self, conf, keyConverter=None, valueConverter=None):
        """
        NotImplemented

        :param conf:
        :param keyConverter:
        :param valueConverter:
        :return:
        """
        raise NotImplementedError

    def saveAsHadoopFile(self, path, outputFormatClass, keyClass=None, valueClass=None, keyConverter=None, valueConverter=None, conf=None, compressionCodecClass=None):
        """
        NotImplemented

        :param path:
        :param outputFormatClass:
        :param keyClass:
        :param valueClass:
        :param keyConverter:
        :param valueConverter:
        :param conf:
        :param compressionCodecClass:
        :return:
        """
        raise NotImplementedError

    def saveAsSequenceFile(self, path, compressionCodecClass=None):
        """
        NotImplemented

        :param path:
        :param compressionCodecClass:
        :return:
        """
        raise NotImplementedError

    def saveAsPickleFile(self, path, batchSize=10):
        """
        NotImplemented

        :param path:
        :param batchSize:
        :return:
        """
        raise NotImplementedError

    def saveAsTextFile(self, path, compressionCodecClass=None):
        """
        NotImplemented

        :param path:
        :param compressionCodecClass:
        :return:
        """
        raise NotImplementedError

    def collectAsMap(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def keys(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def values(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def reduceByKeyLocally(self, func):
        """
        NotImplemented

        :param func:
        :return:
        """
        raise NotImplementedError

    def countByKey(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def join(self, other, numPartitions=None):
        """
        NotImplemented

        :param other:
        :param numPartitions:
        :return:
        """
        raise NotImplementedError

    def leftOuterJoin(self, other, numPartitions=None):
        """
        NotImplemented

        :param other:
        :param numPartitions:
        :return:
        """
        raise NotImplementedError

    def rightOuterJoin(self, other, numPartitions=None):
        """
        NotImplemented

        :param other:
        :param numPartitions:
        :return:
        """
        raise NotImplementedError

    def fullOuterJoin(self, other, numPartitions=None):
        """
        NotImplemented

        :param other:
        :param numPartitions:
        :return:
        """
        raise NotImplementedError

    def partitionBy(self, numPartitions, partitionFunc=None):
        """

        :param numPartitions:
        :param partitionFunc:
        :return:
        """

        return self

    def combineByKey(self, createCombiner, mergeValue, mergeCombiners, numPartitions=None):
        """
        NotImplemented

        :param createCombiner:
        :param mergeValue:
        :param mergeCombiners:
        :param numPartitions:
        :return:
        """
        raise NotImplementedError

    def aggregateByKey(self, zeroValue, seqFunc, combFunc, numPartitions=None):
        """
        NotImplemented

        :param zeroValue:
        :param seqFunc:
        :param combFunc:
        :param numPartitions:
        :return:
        """
        raise NotImplementedError

    def foldByKey(self, zeroValue, func, numPartitions=None):
        """
        NotImplemented

        :param zeroValue:
        :param func:
        :param numPartitions:
        :return:
        """
        raise NotImplementedError

    def _can_spill(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def _memory_limit(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def groupWith(self, other, *others):
        """
        NotImplemented

        :param other:
        :param others:
        :return:
        """
        raise NotImplementedError

    def sampleByKey(self, withReplacement, fractions, seed=None):
        """
        NotImplemented

        :param withReplacement:
        :param fractions:
        :param seed:
        :return:
        """
        raise NotImplementedError

    def subtractByKey(self, other, numPartitions=None):
        """
        NotImplemented

        :param other:
        :param numPartitions:
        :return:
        """
        raise NotImplementedError

    def subtract(self, other, numPartitions=None):
        """
        NotImplemented

        :param other:
        :param numPartitions:
        :return:
        """
        raise NotImplementedError

    def keyBy(self, f):
        """

        :param f:
        :return:
        """
        data = [(f(x), x) for x in self._jrdd]
        return RDD(data, self.ctx)

    def repartition(self, numPartitions):
        """

        :param numPartitions:
        :return:
        """
        return self

    def coalesce(self, numPartitions, shuffle=False):
        """
        NotImplemented

        :param numPartitions:
        :param shuffle:
        :return:
        """
        raise NotImplementedError

    def zipWithUniqueId(self):
        """

        :return:
        """
        data = [(b, a) for a, b in list(enumerate(self._jrdd))]
        return RDD(data, self.ctx)

    def toDebugString(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def getStorageLevel(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def _to_java_object_rdd(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError



# -*- coding: utf-8 -*-

#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import collections

__all__ = ["ResultIterable"]


class ResultIterable(collections.Iterable):

    """
    A special result iterable. This is used because the standard
    iterator can not be pickled
    """

    def __init__(self, data):
        """

        :param data:
        :return:
        """
        self.data = data
        self.index = 0
        self.maxindex = len(data)

    def __iter__(self):
        """

        :return:
        """
        return iter(self.data)

    def __len__(self):
        """

        :return:
        """
        return len(self.data)


from setuptools import setup, find_packages
from codecs import open
from os import path

VERSION = '0.0.4'

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='dummyrdd',
    version=VERSION,
    description='A pure python mocked version of pyspark\'s rdd class',
    long_description=long_description,
    url='https://github.com/wdm0006/dummyrdd',
    download_url='https://github.com/wdm0006/dummyrdd/tarball/' + VERSION,
    license='MIT',
    classifiers=[
      'Development Status :: 3 - Alpha',
      'Intended Audience :: Developers',
      'Programming Language :: Python :: 3',
    ],
    keywords='pyspark spark testing',
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    author='Will McGinnis',
    install_requires=[],
    author_email='will@pedalwrencher.com'
)


# -*- coding: utf-8 -*-

__author__ = 'willmcginnis'


class SQLContext(object):
    def __init__(self, parkContext, sqlContext=None):
        pass

    @property
    def _ssql_ctx(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def setConf(self, key, value):
        """
        NotImplemented

        :param key:
        :param value:
        :return:
        """
        raise NotImplementedError

    def getConf(self, key, defaultValue):
        """
        NotImplemented

        :param key:
        :param defaultValue:
        :return:
        """
        raise NotImplementedError

    def udf(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    def range(self, start, end=None, step=1, numPartitions=None):
        """
        NotImplemented

        :param start:
        :param end:
        :param step:
        :param numPartitions:
        :return:
        """
        raise NotImplementedError

    def registerFunction(self, name, f, returnType=None):
        """
        NotImplemented

        :param name:
        :param f:
        :param returnType:
        :return:
        """
        raise NotImplementedError

    def _inferSchemaFromList(self, data):
        """
        NotImplemented

        :param data:
        :return:
        """
        raise NotImplementedError

    def _inferSchema(self, rdd, samplingRatio=None):
        """
        NotImplemented

        :param rdd:
        :param samplingRatio:
        :return:
        """
        raise NotImplementedError

    def inferSchema(self, rdd, samplingRatio=None):
        """
        NotImplemented

        :param rdd:
        :param samplingRatio:
        :return:
        """
        raise NotImplementedError

    def applySchema(self, rdd, schema):
        """
        NotImplemented

        :param rdd:
        :param schema:
        :return:
        """
        raise NotImplementedError

    def _createFromRDD(self, rdd, schema, samplingRatio):
        """
        NotImplemented

        :param rdd:
        :param schema:
        :param samplingRatio:
        :return:
        """
        raise NotImplementedError

    def _createFromLocal(self, data, schema):
        """
        NotImplemented

        :param data:
        :param schema:
        :return:
        """
        raise NotImplementedError

    def createDataFrame(self, data, schema=None, samplingRatio=None):
        """
        NotImplemented

        :param data:
        :param schema:
        :param samplingRatio:
        :return:
        """
        raise NotImplementedError

    def registerDataFrameAsTable(self, df, tableName):
        """
        NotImplemented

        :param df:
        :param tableName:
        :return:
        """
        raise NotImplementedError

    def parquetFile(self, *paths):
        """
        NotImplemented

        :param paths:
        :return:
        """
        raise NotImplementedError

    def jsonFile(self, path, schema=None, samplingRatio=1.0):
        """
        NotImplemented

        :param path:
        :param schema:
        :param samplingRatio:
        :return:
        """
        raise NotImplementedError

    def jsonRDD(self, rdd, schema=None, samplingRatio=1.0):
        """
        NotImplemented

        :param rdd:
        :param schema:
        :param samplingRatio:
        :return:
        """
        raise NotImplementedError

    def load(self, path=None, source=None, schema=None, **options):
        """
        NotImplemented

        :param path:
        :param source:
        :param schema:
        :param options:
        :return:
        """
        raise NotImplementedError

    def createExternalTable(self, tableName, path=None, source=None, schema=None, **options):
        """
        NotImplemented

        :param tableName:
        :param path:
        :param source:
        :param schema:
        :param options:
        :return:
        """
        raise NotImplementedError

    def sql(self, sqlQuery):
        """
        NotImplemented

        :param sqlQuery:
        :return:
        """
        raise NotImplementedError

    def table(self, tableName):
        """
        NotImplemented

        :param tableName:
        :return:
        """
        raise NotImplementedError

    def tables(self, dbName=None):
        """
        NotImplemented

        :param dbName:
        :return:
        """
        raise NotImplementedError

    def tableNames(self, dbName=None):
        """
        NotImplemented

        :param dbName:
        :return:
        """
        raise NotImplementedError

    def cacheTable(self, tableName):
        """
        NotImplemented

        :param tableName:
        :return:
        """
        raise NotImplementedError

    def uncacheTable(self, tableName):
        """
        NotImplemented

        :param tableName:
        :return:
        """
        raise NotImplementedError

    def clearCache(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError

    @property
    def read(self):
        """
        NotImplemented

        :return:
        """
        raise NotImplementedError
